<?php

namespace Drupal\weather\Form;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\weather\Entity\WeatherDisplayInterface;
use Drupal\weather\Entity\WeatherDisplayPlaceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for the weather_display_place entity edit forms.
 *
 * @ingroup weather_display_place
 */
class WeatherDisplayPlaceForm extends ContentEntityForm {

  /**
   * Weather place storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $weatherPlaceStorage;

  /**
   * WeatherDisplayForm constructor.
   *
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity Type manager service.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface|null $entity_type_bundle_info
   *   The entity type bundle service.
   * @param \Drupal\Component\Datetime\TimeInterface|null $time
   *   The time service.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(EntityRepositoryInterface $entity_repository, EntityTypeManagerInterface $entityTypeManager, EntityTypeBundleInfoInterface $entity_type_bundle_info = NULL, TimeInterface $time = NULL) {
    parent::__construct($entity_repository, $entity_type_bundle_info, $time);

    $this->weatherPlaceStorage = $entityTypeManager->getStorage('weather_place');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.repository'),
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('datetime.time')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $display_type = NULL, $display_number = NULL, WeatherDisplayPlaceInterface $weather_display_place = NULL) {

    if ($display_type == WeatherDisplayInterface::USER_TYPE) {
      $display_number = $this->currentUser()->id();
    }

    $form = parent::buildForm($form, $form_state);

    // If we are on edit form, display type and number not passed here.
    if ($weather_display_place instanceof WeatherDisplayPlaceInterface) {
      $display_type = $weather_display_place->display_type->value;
      $display_number = $weather_display_place->display_number->value;
    }

    // If the place exists, get the configuration.
    // If it does not exist - get the default place configuration.
    $settings = $this->getLocationSettings($weather_display_place);

    $form['search'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Weather location'),
      '#autocomplete_route_name' => 'weather.settings.places.autocomplete',
      '#placeholder' => $this->t('Please start typing here to see the suggestions'),
      '#description' => $this->t('We are using Google Places Auto-complete API to show the suggestion.')
    ];

    $form['displayed_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Weather location'),
      '#placeholder' => $this->t('This field will be automatically populated with a found location.'),
      '#default_value' => $settings['displayed_name'],
      '#description' => $this->t('This field will be automatically populated with a found location. However you can change it to desired output.')
    ];

    $form['lat'] =  [
      '#type' => 'textfield',
      '#title' => $this->t('Latitude'),
      '#default_value' => $settings['lat'],
    ];

    $form['lon'] =  [
      '#type' => 'textfield',
      '#title' => $this->t('Longitude'),
      '#default_value' => $settings['lon'],
    ];

    $form['geoid'] =  [
      '#type' => 'textfield',
      '#title' => $this->t('Geoname ID'),
      '#default_value' => $settings['geoid'],
    ];

    $form['weight'] = [
      '#type' => 'weight',
      '#title' => $this->t('Weight'),
      '#default_value' => $settings['weight'],
      '#description' => $this->t('Optional. In the block, the heavier locations will sink and the lighter locations will be positioned nearer the top. Locations with equal weights are sorted alphabetically.'),
    ];
    $form['display_type'] = [
      '#type' => 'value',
      '#value' => $display_type,
    ];
    $form['display_number'] = [
      '#type' => 'value',
      '#value' => $display_number,
    ];

    // Use different path for delete form for non-admin user.
    if ($display_type == WeatherDisplayInterface::USER_TYPE && $weather_display_place instanceof WeatherDisplayPlaceInterface) {
      $form["actions"]["delete"]["#url"] = Url::fromRoute('weather.user.weather_display_place.delete_form', [
        'user' => $display_number,
        'weather_display_place' => $weather_display_place->id(),
      ]);
    }

    $form['#attached']['library'][] = 'weather/weather.places_helper';
    $form['#attached']['drupalSettings']['google']['autocomplete']['places'] = [];
    $form['#attached']['drupalSettings']['google']['auth'] = $this->config('weather.settings')->get('gmaps_api_key') ?? '';
    $form['#attached']['drupalSettings']['google']['geoname_uid'] = $this->config('weather.settings')->get('geonames_api_username') ?? '';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $status = parent::save($form, $form_state);

    // Invalidate related weather display cache, once place is saved.
    $display_type = $this->entity->display_type->value;
    $display_number = $this->entity->display_number->value;
    $displays = $this->entityTypeManager->getStorage('weather_display')
      ->loadByProperties([
        'type' => $display_type,
        'number' => $display_number,
      ]);
    foreach ($displays as $display) {
      $tags = $display->getCacheTags();

      // Separately invalidate cache_tag for user.
      // For correct page update after adding new location.
      $tags[] = 'weather_display:' . $this->currentUser()->id();
      Cache::invalidateTags($tags);
    }

    // Show message.
    if ($status == SAVED_NEW) {
      $message = $this->t('Added new place to weather display');
    }
    else {
      $message = $this->t('Updated existing place in weather display');
    }
    $this->messenger()->addStatus($message);

    // Fulfill place data into DB.
    $place = transliterator_create('Any-Latin; Latin-ASCII')->transliterate($form_state->getValue('displayed_name'));
    $place = str_replace([',', ' '], '_', $place);
    if(empty($this->weatherPlaceStorage->load($form_state->getValue('geoid')))) {
      $this->weatherPlaceStorage->create([
        'geoid' => $form_state->getValue('geoid'),
        'latitude' => $form_state->getValue('lat'),
        'longitude' => $form_state->getValue('lon'),
        'country' => 'Location',
        'name' => $form_state->getValue('displayed_name'),
        'link' => 'detailed-forecast/' . $place,
      ])->save();
    }

    switch ($this->entity->display_type->value) {
      case WeatherDisplayInterface::USER_TYPE:
        $form_state->setRedirectUrl(Url::fromRoute('weather.user.settings', ['user' => $this->entity->display_number->value]));
        break;

      default:
        $form_state->setRedirectUrl(Url::fromRoute('weather.settings'));
        break;
    }

    return $status;
  }

  /**
   * Finds location settings for display Place form.
   *
   * @param \Drupal\weather\Entity\WeatherDisplayPlaceInterface|null $weather_display_place
   *   Weather display place entity.
   *
   * @return array
   *   Settings.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getLocationSettings(WeatherDisplayPlaceInterface $weather_display_place = NULL) {
    // Set defaults.
    $settings = [
      'geoid' => 'geonames_703448',
      'displayed_name' => 'Kyiv',
      'weight' => 0,
      'lat' => '',
      'lon' => '',
    ];

    if ($weather_display_place instanceof WeatherDisplayPlaceInterface) {
      foreach ($settings as $field_name => $value) {
        if ($weather_display_place->hasField($field_name)) {
          $type = $weather_display_place->get($field_name)
            ->getFieldDefinition()
            ->getType();
          if ($type == 'entity_reference') {
            $settings[$field_name] = $weather_display_place->{$field_name}->target_id;
          }
          else {
            $settings[$field_name] = $weather_display_place->{$field_name}->value;
          }
        }
      }

      // Find related country.
      $place = $this->entityTypeManager->getStorage('weather_place')
        ->load($settings['geoid']);
      $settings['country'] = $place->country->value;
    }

    return $settings;
  }


}
